#include <amdis/AMDiS.hpp>
#include <amdis/AdaptStationary.hpp>
#include <amdis/LocalOperators.hpp>
#include <amdis/ProblemStat.hpp>
#include <amdis/common/QuadMath.hpp>

#include <dune/common/version.hh>
#include <dune/grid/yaspgrid.hh>

using namespace AMDiS;

template <class T>
void test()
{
  // use T as coordinate type
  using Grid = Dune::YaspGrid<2, Dune::EquidistantCoordinates<T,2>>;
  Grid grid({T(1), T(1)}, {8,8});

  // use T as range type for basis
  using namespace Dune::Functions::BasisFactory;
#if DUNE_VERSION_LT(DUNE_FUNCTIONS,2,7)
  auto basis = makeBasis(grid.leafGridView(), power<1>(lagrange<1>(), flatLexicographic()));
#else
  auto basis = makeBasis(grid.leafGridView(), power<1>(lagrange<1,T>(), flatLexicographic()));
#endif
  using Basis = decltype(basis);

  // use T as coefficient type
  using Param = DefaultProblemTraits<Basis, T>;

  ProblemStat<Param> prob("ellipt", grid, basis);
  prob.initialize(INIT_ALL);
  prob.boundaryManager()->setBoxBoundary({1,1,2,2});

  prob.addMatrixOperator(sot(T(1)), 0, 0);
  prob.addVectorOperator(zot(T(1)), 0);
  prob.addDirichletBC(BoundaryType{1}, 0,0, T(0));


  AdaptInfo adaptInfo("adapt");

  // test copy constructor of problem
  auto prob2(prob);
  AdaptStationary adaptStat2("adapt", prob2, adaptInfo);
  adaptStat2.adapt();

  // test move constructor of problem
  auto prob3(std::move(prob2));
  AdaptStationary adaptStat3("adapt", prob3, adaptInfo);
  adaptStat3.adapt();
}

int main(int argc, char** argv)
{
  Environment env(argc, argv);

#if !HAVE_PETSC || PETSC_USE_REAL_SINGLE
  test<float>();
#endif

#if !HAVE_PETSC || PETSC_USE_REAL_DOUBLE
  test<double>();
#endif

#if !HAVE_PETSC
  test<long double>();
#endif

#if HAVE_QUADMATH && (!HAVE_PETSC || PETSC_USE_REAL___FLOAT128)
  test<Dune::Float128>();
#endif

  return 0;
}
